import { defineComponent, ref, reactive, onMounted, watch, toRef, onUnmounted, h, Teleport } from 'vue';

function toVueComponent(HFC, lowercase) {
    var propKeys = Object.keys(HFC.interface.attrs || {});
    var eventKeys = Object.keys(HFC.interface.events || {});
    var slotKeys = Object.keys(HFC.interface.slots || {});
    var propKeyMap = {};
    var eventKeyMap = {};
    var slotKeyMap = {};
    if (lowercase) {
        propKeys.forEach(function (k) { return (propKeyMap[k.toLowerCase()] = k); });
        eventKeys.forEach(function (k) {
            return (eventKeyMap["on" + k[0].toUpperCase() + k.slice(1).toLowerCase()] = k);
        });
        slotKeys.forEach(function (k) { return (slotKeyMap[k.toLowerCase()] = k); });
    }
    else {
        propKeys.forEach(function (k) { return (propKeyMap[k] = k); });
        eventKeys.forEach(function (k) { return (eventKeyMap["on" + k[0].toUpperCase() + k.slice(1)] = k); });
        slotKeys.forEach(function (k) { return (slotKeyMap[k] = k); });
    }
    var observePropKeys = Object.keys(propKeyMap);
    var observeEventKeys = Object.keys(eventKeyMap);
    var observeSlotKeys = Object.keys(slotKeyMap);
    return defineComponent({
        props: observePropKeys,
        setup: function (vueProps, _a) {
            var vueSlots = _a.slots, vueAttrs = _a.attrs;
            var container = ref();
            var events = {};
            observeEventKeys.forEach(function (key) {
                var event = vueAttrs[key];
                if (!event)
                    return;
                events[eventKeyMap[key]] = event;
            });
            var slots = {};
            var renderedSlots = reactive({});
            observeSlotKeys.forEach(function (key) {
                var slot = vueSlots[key];
                if (!slot)
                    return;
                slots[slotKeyMap[key]] = function (container, args) {
                    var slotItem = renderedSlots[key];
                    if (!slotItem) {
                        renderedSlots[key] = { name: key, container: container, slot: slot, args: args };
                        return;
                    }
                    if (!container) {
                        delete renderedSlots[key];
                        return;
                    }
                    slotItem.container = container;
                    slotItem.args = args;
                };
            });
            var hfc;
            onMounted(function () {
                var attrs = {};
                observePropKeys.forEach(function (key) {
                    if (vueProps[key])
                        attrs[propKeyMap[key]] = vueProps[key];
                });
                hfc = new HFC(container.value, {
                    attrs: attrs,
                    events: events,
                    slots: slots,
                });
                hfc.connectedCallback();
                observePropKeys.forEach(function (key) {
                    watch(toRef(vueProps, key), function (curr, prev) {
                        hfc.attributeChangedCallback("attr", propKeyMap[key], prev, curr);
                    }, { deep: true });
                });
            });
            onUnmounted(function () {
                hfc.disconnectedCallback();
            });
            return function () { return [
                h("div", { ref: container }),
                Object.keys(renderedSlots).map(function (key) {
                    var item = renderedSlots[key];
                    return h(Teleport, { to: item.container }, item.slot(item.args));
                }),
            ]; };
        },
    });
}

export { toVueComponent as default };
